#include "StatePlay.h"
#include "TextureHelper.h"

#include "WorldGenerator.h"

#include "Blocks.h"

StatePlay::StatePlay()
{	
	mRender = NULL;
	mSystemMgr = NULL;
	fppCam = NULL;

	//utils
	freeMemory = 0;
	freeMemoryTimer = 0.0f;

	showCube = true;

	//zmienne do poruszania
	GRAVITY = -9.8f;
	JUMPVELOCITY = 4.0f;
	CLIMBVELOCITY = 2.5f;

	walkingOnGround = false;
	jumping = false;
	headInWater = false;
	footInWater = false;

	selectedCube = 0;
	selectedCubeSet = 0;
	selectedCubeStart = 0;

	ram1 = 0;
	ram2 = 0;
	dt = 0.0f;
	sunTime = 21600.0f;//6 am
	sunTimeTick = 0.0f;
	sunMoonSwitch = true;

	cameraSpeed = 2.0f / 60.0f;
	cameraMoveSpeed = 4.0f/ 60.0f;

	//stete to game
	menuState = 0;
	selectPos = 0;

	//flying
	canFly = false;
	devMode = false;
	makeScreen = false;
}

StatePlay::~StatePlay()
{

}

void StatePlay::InitCamera()
{
	fppCam = new Camera();
	fppCam->PositionCamera(playerPosition.x,playerPosition.y,playerPosition.z, playerPosition.x,playerPosition.y,playerPosition.z-5.0f, 0.0f,1.0f,0.0f);
	mRender->SetActiveCamera(fppCam);
}

void StatePlay::Init()
{
	initBlocks();
	//set render manager instance
	mRender = RenderManager::InstancePtr();
	mSystemMgr = SystemManager::Instance();
	mSoundMgr = SoundManager::Instance();
	mIhelper = InputHelper::Instance();

	playerPosition = newPlayerPos = oldPlayerPos = Vector3(64.0f,90.0f,64.0f);

	WorldGenerator *mGen = new WorldGenerator();
	
	//then create our perfect world
	mWorld = new CraftWorld();
	mWorld->initWorld(128, 16);
	mGen->initRandompMap(128, 16, mWorld);	
	delete mGen;
	//mWorld->initRandompMap(128,16);
	mWorld->setTextureSize(128,16);
	mWorld->initWorldBlocksLight();
	mWorld->SetWolrdTime(5);
	mWorld->UpdatePlayerZoneBB(playerPosition);
	mWorld->buildMap();
	mWorld->buildblocksVerts();

	dt = mTimer.GetDeltaTime();

	//block sets info
	allcubes = mWorld->GetBlockTypesCount();
	//selectedCubeEnd = allcubes - 2;//because we don't want first one and last one
	cubesSets = std::floor(allcubes / 9);//9 cubes is set

	LoadTextures();

	menuOptions = false;
	optionsMenuPos = 0;

	analogLeft = analogRight = analogUp = analogDown = false;

	walkSoundAccu = 0.0f;
	isWalking = false;
}

void StatePlay::InitParametric(int terrainType,bool makeFlat,bool makeTrees,bool makeWater,bool makeCaves)
{
	initBlocks();
	//set render manager instance
	mRender = RenderManager::InstancePtr();
	mSystemMgr = SystemManager::Instance();
	mSoundMgr = SoundManager::Instance();
	mIhelper = InputHelper::Instance();

	//
	playerPosition = newPlayerPos = oldPlayerPos = Vector3(64.0f,90.0f,64.0f);

	//then create our perfect world
	mWorld = new CraftWorld();
	mWorld->initWorld(128, 16);
	
	WorldGenerator *mGen = new WorldGenerator();
	mGen->initParametricRandompMap(128,16, mWorld, terrainType,makeFlat,makeTrees,makeWater,makeCaves);
	delete mGen;
	//mWorld->initRandompMap(128,16,terrainType,makeFlat,makeTrees,makeWater,makeCaves);
	mWorld->setTextureSize(128,16);
	mWorld->initWorldBlocksLight();
	mWorld->SetWolrdTime(5);
	mWorld->UpdatePlayerZoneBB(playerPosition);
	mWorld->buildMap();
	mWorld->buildblocksVerts();

	dt = mTimer.GetDeltaTime();

	//block sets info
	allcubes = mWorld->GetBlockTypesCount();
	//selectedCubeEnd = allcubes - 2;//because we don't want first one and last one
	cubesSets = std::floor(allcubes / 9);//9 cubes is set

	LoadTextures();

	menuOptions = false;
	optionsMenuPos = 0;

	analogLeft = analogRight = analogUp = analogDown = false;

	walkSoundAccu = 0.0f;
	isWalking = false;
}

void StatePlay::SetWorldAndSaveName(std::string worldName,std::string fileName)
{
	if(mWorld != NULL)
	{
		sprintf(mWorld->worldName,"%s",worldName.c_str());
	}
	saveFileName = fileName;
}

void StatePlay::LoadMap(std::string fileName,bool compressed)
{
	//set render manager instance
	mRender = RenderManager::InstancePtr();
	mSystemMgr = SystemManager::Instance();
	mSoundMgr = SoundManager::Instance();
	mIhelper = InputHelper::Instance();

	//save name
	saveFileName = fileName;
	//
	playerPosition = newPlayerPos = oldPlayerPos = Vector3(64.0f,90.0f,64.0f);

	//then create our perfect world
	mWorld = new CraftWorld();

	if(compressed)
	{
		mWorld->LoadCompressedWorld(saveFileName);
		//set player pos from map
		playerPosition = mWorld->GetPlayerPos();
		newPlayerPos  = mWorld->GetPlayerPos();
		oldPlayerPos = mWorld->GetPlayerPos();
		mWorld->SetWolrdTime(mWorld->worldDayTime);
	}
	else
	{
		mWorld->LoadWorld(saveFileName.c_str());
		mWorld->SetWolrdTime(5);
	}

	mWorld->setTextureSize(128,16);
	mWorld->UpdatePlayerZoneBB(playerPosition);
	mWorld->buildMap();
	mWorld->buildblocksVerts();

	dt = mTimer.GetDeltaTime();

	//block sets info
	allcubes = mWorld->GetBlockTypesCount();
	//selectedCubeEnd = allcubes - 2;//because we don't want first one and last one
	cubesSets = std::floor(allcubes / 9);//9 cubes is set

	LoadTextures();

	menuOptions = false;
	optionsMenuPos = 0;

	analogLeft = analogRight = analogUp = analogDown = false;

	walkSoundAccu = 0.0f;
	isWalking = false;

	SetDayTimeAfterLoad();
}

void StatePlay::LoadTextures()
{
	//terrain texure
	texture = TextureHelper::Instance()->GetTexture(TextureHelper::Terrain);

	//water filter
	blue = TextureHelper::Instance()->GetTexture(TextureHelper::Blue);

	//bar image
	barSprite = new Sprite(TextureHelper::Instance()->GetTexture(TextureHelper::Utils),0,0,182,22);
	barSprite->SetPosition(240,253);
	barSprite->Scale(1.75f,1.75f);

	selectSprite = new Sprite(TextureHelper::Instance()->GetTexture(TextureHelper::Utils),1,23,22,22);
	selectSprite->SetPosition(100,253);
	selectSprite->Scale(1.75f,1.75f);

	crossSprite = new Sprite(TextureHelper::Instance()->GetTexture(TextureHelper::Utils),201,13,9,9);
	crossSprite->SetPosition(240,136);
	crossSprite->Scale(2.0f,2.0f);

	buttonSprite = new Sprite(TextureHelper::Instance()->GetTexture(TextureHelper::Utils),24,22,200,20);
	buttonSprite->SetPosition(240,150);

	sbuttonSprite = new Sprite(TextureHelper::Instance()->GetTexture(TextureHelper::Utils),24,42,200,20);
	sbuttonSprite->SetPosition(240,150);

	cubeModel = new ObjModel();
	cubeModel->LoadObj("Assets/Lamecraft/textureCube.obj");
	cubeModel->Optimize();

	//sky dome
	skyDome = new SkyDome();
	skyDome->CreateSkyDomeMesh();

	skyDome->SetTexture(TextureHelper::Instance()->GetTexture(TextureHelper::Sky));
	skyDome->timeOfDay = 0.1f;

	skyLight = new SkyLight();
	skyLight->SetTexture(TextureHelper::Instance()->GetTexture(TextureHelper::Sun));
}

void StatePlay::SetDayTimeAfterLoad()
{
	if(mWorld->worldVersion >= 3)
	{
		if(mWorld->worldDayTime >= 5.0f && mWorld->worldDayTime < 21.0f)
		{
			skyLight->SetTexture(TextureHelper::Instance()->GetTexture(TextureHelper::Sun));
			sunMoonSwitch = true;
		}else
		{
			skyLight->SetTexture(TextureHelper::Instance()->GetTexture(TextureHelper::Moon));
			sunMoonSwitch = false;
		}

		sunTime = mWorld->sunTime;
		skyDome->timeOfDay = mWorld->worldDayTime * 0.041666f;
	}
}

void StatePlay::Enter()
{
	RenderManager::InstancePtr()->SetPerspective(55.0f, 480.0f / 272.0f, 0.18f, 1000.f);
}

void StatePlay::CleanUp()
{
	delete mRender->mCam;
	mRender->mCam = NULL;

	delete buttonSprite;
	delete sbuttonSprite;
	delete barSprite;
	delete selectSprite;
	delete crossSprite;
	delete cubeModel;
	delete skyDome;
	delete skyLight;
	delete mWorld;
	deleteBlocks();
}

void StatePlay::Pause()
{

}

void StatePlay::Resume()
{

}

void StatePlay::HandleEvents(StateManager* sManager)
{
	//update delta time
	dt = mTimer.GetDeltaTime();

	cameraSpeed = 2.0f * dt;
	cameraMoveSpeed = 4.0f * dt;

	//free memory counter
	if(devMode)
	{
		freeMemoryTimer += dt;
		if(freeMemoryTimer > 1.0f)
		{
			freeMemory = mSystemMgr->ramAvailable();
			freeMemoryTimer = 0.0f;
		}
	}

	//update input
	mSystemMgr->InputUpdate();

	if(menuState == 0)//game state
	{
		//menu
		if(keyPressed(InputHelper::Instance()->getButtonToAction(15)))
		{
			menuState = 1;
			menuOptions = false;
			optionsMenuPos = 0;
		}

		//camera input
		//rotate down
		if(keyHold(InputHelper::Instance()->getButtonToAction(5)))
		{
			fppCam->PitchView(-cameraSpeed);
		}
		//rotate up
		if(keyHold(InputHelper::Instance()->getButtonToAction(4)))
		{
			fppCam->PitchView(cameraSpeed);
		}
		//rotate right
		if(keyHold(InputHelper::Instance()->getButtonToAction(7)))
		{
			fppCam->RotateView(-cameraSpeed,0,1,0);
		}
		//rotate left
		if(keyHold(InputHelper::Instance()->getButtonToAction(6)))
		{
			fppCam->RotateView(cameraSpeed,0,1,0);
		}

		//move right
		if(keyHold(InputHelper::Instance()->getButtonToAction(3)))
		{
			fppCam->StrafePhysic(cameraMoveSpeed);
		}
		//move left
		if(keyHold(InputHelper::Instance()->getButtonToAction(2)))
		{
			fppCam->StrafePhysic(-cameraMoveSpeed);
		}

		//move back
		if(keyHold(InputHelper::Instance()->getButtonToAction(1)))
		{
			if(canFly)
				fppCam->MovePhysic(-cameraMoveSpeed);
			else
				fppCam->MovePhysicNoY(-cameraMoveSpeed);
		}

		//move forward
		if(keyHold(InputHelper::Instance()->getButtonToAction(0)))
		{
			if(canFly)
				fppCam->MovePhysic(cameraMoveSpeed);
			else
				fppCam->MovePhysicNoY(cameraMoveSpeed);
		}

		//cubes management
		//switch right
		if(keyPressed(InputHelper::Instance()->getButtonToAction(9)))
		{
			selectedCube++;
			if(selectedCubeSet == cubesSets * 9)
			{
				if(selectedCube > (allcubes - (cubesSets * 9) - 3))
					selectedCube = 0;
			}else
			{
				if(selectedCube > 8)
					selectedCube = 0;
			}

			selectSprite->SetPosition(100 + (selectedCube * 35),253);
		}
		//switch right
		if(keyPressed(InputHelper::Instance()->getButtonToAction(8)))
		{
			selectedCube--;
			if(selectedCube < 0)
			{
				if(selectedCubeSet == cubesSets * 9)
					selectedCube = allcubes - (cubesSets * 9) - 3;
				else
					selectedCube = 8;
			}

			selectSprite->SetPosition(100 + (selectedCube * 35),253);
		}
		//switch down
		if(keyPressed(InputHelper::Instance()->getButtonToAction(11)))
		{
			selectedCubeSet-=9;
			if(selectedCubeSet < 0)
			{
				selectedCubeSet = cubesSets * 9;

				//don't allow to select empty block
				if(selectedCube > (allcubes - (cubesSets * 9) - 3))
				{
					selectedCube = allcubes - (cubesSets * 9) - 3;
					selectSprite->SetPosition(100 + (selectedCube * 35),253);
				}
			}
		}
		//switch up
		if(keyPressed(InputHelper::Instance()->getButtonToAction(10)))
		{
			selectedCubeSet+=9;

			if(selectedCubeSet == cubesSets * 9)
			{
				//don't allow to select empty block
				if(selectedCube > (allcubes - (cubesSets * 9) - 3))
				{
					selectedCube = allcubes - (cubesSets * 9) - 3;
					selectSprite->SetPosition(100 + (selectedCube * 35),253);
				}
			}

			if(selectedCubeSet > (cubesSets * 9))
				selectedCubeSet = 0;
		}

		//add cube
		if(keyPressed(InputHelper::Instance()->getButtonToAction(14)))
		{
			//add cube
			Vector3 rayDir = fppCam->m_vView - fppCam->m_vPosition;
			rayDir.normalize();

			//get position and view vector
			Vector3 testPos;

			//move to the target +=0.5
			for(float i = 0;i < 5.25f;i+=0.25f)
			{
				testPos = fppCam->m_vPosition + (rayDir * i);

				//check if we are touch something
				if(mWorld->BlockEditable(testPos.x,testPos.y,testPos.z))
				{
					BoundingBox testBox = BoundingBox(Vector3(cubePos.x - 0.5f,cubePos.y - 0.5f,cubePos.z - 0.5f),Vector3(cubePos.x + 0.5f,cubePos.y + 0.5f,cubePos.z + 0.5f));
					Ray tesRay = Ray(fppCam->m_vPosition,rayDir);

					float distance = -1.0f;
					tesRay.hasIntersected(testBox,distance);

					if(distance != -1.0f)
					{
						Vector3 hitPos = fppCam->m_vPosition + (rayDir * distance);
						Vector3 normal = (hitPos - cubePos);
						normal.normalize();

						Vector3 testPos2;

						if(normal.x < -0.66f)
							testPos2.x = -1.0f;
						else if(normal.x > 0.66f)
							testPos2.x = 1.0f;
						else
							testPos2.x = 0.0f;

						if(normal.y < -0.66f)
							testPos2.y = -1.0f;
						else if(normal.y > 0.66f)
							testPos2.y = 1.0f;
						else
							testPos2.y = 0.0f;

						if(normal.z < -0.66f)
							testPos2.z = -1.0f;
						else if(normal.z > 0.66f)
							testPos2.z = 1.0f;
						else
							testPos2.z = 0.0f;


						testPos2 += testPos;

						//check if this block is empty
						if(mWorld->GetBlock(testPos2.x,testPos2.y,testPos2.z) == 0)
						{
							//now check if player is not there ;) we don't want to place cubes where we stand
							int boxX = (int)testPos2.x;
							int boxY = (int)testPos2.y;
							int boxZ = (int)testPos2.z;

							BoundingBox blockBox = BoundingBox(Vector3(boxX,boxY,boxZ),Vector3(boxX + 1,boxY + 1,boxZ + 1));
							BoundingBox playerBox = BoundingBox(Vector3(playerPosition.x - 0.15f,playerPosition.y - 1.5f,playerPosition.z - 0.15f),Vector3(playerPosition.x + 0.15f,playerPosition.y + 0.2f,playerPosition.z + 0.15f));

							if(!blockBox.intersect(playerBox))
							{
								//check if you want put light source or normal block
								if(mWorld->CanPutBlockHere(testPos2.x,testPos2.y,testPos2.z,selectedCubeSet + selectedCube+1))
								{
									if(mWorld->LightSourceBlock(selectedCubeSet + selectedCube+1))
									{
										mWorld->SetLigtSourcePosition(testPos2.x,testPos2.y,testPos2.z,selectedCubeSet + selectedCube+1);
									}

									mWorld->GetBlock(testPos2.x,testPos2.y,testPos2.z) = selectedCubeSet + selectedCube+1;//set block type

									int chunkTarget = mWorld->getChunkId(testPos2);

									if(chunkTarget != -1)
									{
										mSoundMgr->PlayPlopSound();

										//rebuild
										if(mWorld->LightSourceBlock(selectedCubeSet + selectedCube+1))
										{
											mWorld->RebuildChunksLight(testPos2,chunkTarget,(selectedCubeSet + selectedCube+1));
										}
										else
										{
											//check if there are light sources nearby
											mWorld->UpdateLightAreaIn(testPos2);

											//rebuild chunks
											mWorld->rebuildChunk(chunkTarget);
											mWorld->rebuildTransparentChunk(chunkTarget);
											//rebuild chunks that are near this chunk
											mWorld->rebuildNearestChunks(chunkTarget,testPos2);
										}
									}
								}

								fppCam->needUpdate = true;
							}
						}
					}

					break;
				}
			}

		}

		//remove cube
		if(keyPressed(InputHelper::Instance()->getButtonToAction(13)))
		{
			bool wasLight = false;
			block_t oldBlock = 0;
			//remove cube
			Vector3 rayDir = fppCam->m_vView - fppCam->m_vPosition;
			rayDir.normalize();

			//we are takin our positiona and view vector
			Vector3 testPos;

			//we are moving slowly to the target +=0.5
			for(float i = 0;i < 5.25f;i+=0.25f)
			{
				testPos = fppCam->m_vPosition + (rayDir * i);

				//check if we touch something
				if(mWorld->BlockEditable(testPos.x,testPos.y,testPos.z))
				{
					//check if it's light source
					if(mWorld->LightSourceBlock(mWorld->GetBlock(testPos.x,testPos.y,testPos.z)))//if it's light block
					{
						oldBlock = mWorld->GetBlock(testPos.x,testPos.y,testPos.z);
						wasLight = true;
					}

					//remove block
					mSoundMgr->PlayWalkSound(mWorld->BlockSoundAtPos(testPos));
					mWorld->GetBlock(testPos.x,testPos.y,testPos.z) = 0;
					int	chunkTarget = mWorld->getChunkId(testPos);

					//check if this block is a support for light block
					mWorld->CheckForTorchSupport(testPos.x,testPos.y,testPos.z,mWorld->GetBlock(testPos.x,testPos.y,testPos.z));


					//before rebuilding update light info
					if(wasLight)
					{
						mWorld->RemoveLigtSourceAtPosition(testPos.x,testPos.y,testPos.z,oldBlock);
					}

					if(chunkTarget != -1)
					{
						//rebuild
						if(wasLight)
						{
							mWorld->RebuildChunksLight(testPos,chunkTarget,oldBlock);
						}else
						{
							//check if there are light sources nearby
							mWorld->UpdateLightAreaIn(testPos);

							//rebuild chunks
							mWorld->rebuildChunk(chunkTarget);
							mWorld->rebuildTransparentChunk(chunkTarget);
							mWorld->rebuildNearestChunks(chunkTarget,testPos);
						}
					}

					fppCam->needUpdate = true;
					break;
				}
			}
		}

		//jump
		if(keyHold(InputHelper::Instance()->getButtonToAction(12)))
		{
			if(walkingOnGround || headInWater)
			{
				jumping = true;
			}
		}
	}else if(menuState == 1)//menu state
	{
		//turn off menu with the same key
		if(keyPressed(InputHelper::Instance()->getButtonToAction(15)))
		{
			menuState = 0;
			menuOptions = false;
			optionsMenuPos = 0;
			selectPos = 0;
		}

		//menu keys
		if(menuOptions)
		{
			//up, down
			if(mSystemMgr->KeyPressed(PSP_CTRL_UP))
			{
				optionsMenuPos--;
				if(optionsMenuPos < 0)
					optionsMenuPos = 6;

				mSoundMgr->PlayMenuSound();
			}

			if(mSystemMgr->KeyPressed(PSP_CTRL_DOWN))
			{
				optionsMenuPos++;
				if(optionsMenuPos > 6)
					optionsMenuPos = 0;

				mSoundMgr->PlayMenuSound();
			}

			//back
			if(mSystemMgr->KeyPressed(PSP_CTRL_CIRCLE))
			{
				selectPos = 0;
				menuOptions = false;
			}

			if(mSystemMgr->KeyPressed(PSP_CTRL_CROSS))
			{
				//fly
				if(optionsMenuPos == 0)
					canFly = !canFly;

				//devmode
				if(optionsMenuPos == 1)
					devMode = !devMode;

				//take screenshot
				if(optionsMenuPos == 2)
					makeScreen = true;

				//rename
				if(optionsMenuPos == 3)
				{
					char description[128];
					char starting[11] = {'W','o','r','l','d',' ','n','a','m','e','\0'};
					int lenght = 0;
					mRender->SetOrtho(0,0,0,0,0,0);
					if(mSystemMgr->ShowOSKDanzeff(starting,description,lenght) != -1)
					{
						std::string newWorldName = description;

						sprintf(mWorld->worldName,"%s",newWorldName.c_str());
					}
				}

				if(optionsMenuPos == 4)
					mSoundMgr->playerSounds = !mSoundMgr->playerSounds;

				if(optionsMenuPos == 5)
					mWorld->freezeDayTime = !mWorld->freezeDayTime;

				//back to ingame menu
				if(optionsMenuPos == 6)
				{
					selectPos = 0;
					menuOptions = false;
				}
			}
		}else
		{
			//up, down
			if(mSystemMgr->KeyPressed(PSP_CTRL_UP))
			{
				selectPos--;
				if(selectPos < 0)
					selectPos = 4;

				mSoundMgr->PlayMenuSound();
			}

			if(mSystemMgr->KeyPressed(PSP_CTRL_DOWN))
			{
				selectPos++;
				if(selectPos > 4)
					selectPos = 0;

				mSoundMgr->PlayMenuSound();
			}

			if(mSystemMgr->KeyPressed(PSP_CTRL_CIRCLE))
			{
				menuState = 0;
				selectPos = 0;
				optionsMenuPos = 0;
				menuOptions = false;
			}

			if(mSystemMgr->KeyPressed(PSP_CTRL_CROSS))
			{
				if(selectPos == 0)//resume
				{
					menuState = 0;
				}
				if(selectPos == 1)//options
				{
					optionsMenuPos = 0;
					menuOptions = true;
				}
				if(selectPos == 2)//save
				{
					mWorld->sunTime = sunTime;
					mWorld->SaveCompressedWorld(saveFileName.c_str());
					menuState = 0;
				}
				if(selectPos == 3)//save end exit
				{
					//save
					mWorld->sunTime = sunTime;
					mWorld->SaveCompressedWorld(saveFileName.c_str());
					//exit
					sManager->PopState();
				}
				if(selectPos == 4)//exit
				{
					//exit
					sManager->PopState();
				}
			}
		}
	}
}

void StatePlay::Update(StateManager* sManager)
{
	if(menuState == 0)//game state
	{
		if(fppCam->needUpdate)
		{
			showCube = false;

			//show pick cube
			Vector3 rayDir = fppCam->m_vView - fppCam->m_vPosition;
			rayDir.normalize();

			//we take position and view vector
			Vector3 testPos ;

			//moving slowly to the target
			for(float i = 0;i < 5.25f;i+=0.25f)
			{
				testPos = fppCam->m_vPosition + (rayDir * i);

				//check if we touching something
				if(mWorld->BlockEditable(testPos.x,testPos.y,testPos.z))
				{
					//if yes then show pick cube
					cubePos.x = (int)testPos.x + 0.5f;
					cubePos.y = (int)testPos.y + 0.5f;
					cubePos.z = (int)testPos.z + 0.5f;

					showCube = true;
					break;
				}
			}

		}

		int soundBlockType = -1;

		//update player position
		if(dt < 0.1f)
		{
			Vector3 delta = fppCam->m_vView - fppCam->m_vPosition;
			playerPosition = fppCam->m_vPosition;

			//gravity
			Vector3 footPosition = playerPosition + Vector3(0.0f, -1.5f, 0.0f);
			Vector3 legsPosition = playerPosition + Vector3(0.0f, -1.0f, 0.0f);
			Vector3 headPosition = playerPosition + Vector3(0.0f, 0.1f, 0.0f);

			//foot in water
			if(mWorld->PlayerInWater(legsPosition))
				footInWater = true;
			else
				footInWater = false;

			//check if head is is in the water
			if(mWorld->PlayerInWater(headPosition))
			{
				//change gravity
				playerVelocity.y += (GRAVITY/8.0f) * dt;
				headInWater = true;
			}else
			{
				headInWater = false;
				playerVelocity.y += GRAVITY * dt;
			}



			//check if we are standing on ground
			if(mWorld->SolidAtPointForPlayer(footPosition) || mWorld->SolidAtPointForPlayer(headPosition))
			{
				walkingOnGround = true;
				soundBlockType = mWorld->BlockSoundAtPos(footPosition);

				// If the player has their head stuck in a block, push them down.
				if (mWorld->SolidAtPointForPlayer(headPosition))
				{
					int blockIn = (int)(headPosition.y);
					playerPosition.y = (float)(blockIn - 0.15f);
				}

				// If the player is stuck in the ground, bring them out.
				// This happens because we're standing on a block at -1.5, but stuck in it at -1.4, so -1.45 is the sweet spot.
				if (mWorld->SolidAtPointForPlayer(footPosition))
				{
					int blockOn = (int)(footPosition.y);
					playerPosition.y = (float)(blockOn + 1 + 1.45f);
				}

				playerVelocity.y = 0.0f;

				//dodatkowa logika podczas stania na klocku
				//jump
				if(jumping)
				{
					playerVelocity.y = 1.2f * JUMPVELOCITY;
					jumping = false;
					walkingOnGround = false;
				}

			}else
			{
				walkingOnGround = false;
			}

			//update position
			if(!canFly)
				playerPosition += playerVelocity * dt;
			else
				playerVelocity.y = 0.0f;


			//collision with walls
			isWalking = false;
			Vector3 moveVector = fppCam->m_vVelocity;

			if(moveVector.x != 0.0f || moveVector.z != 0.0f)
			{
				if(TryToMove(moveVector,dt))
				{
					isWalking = true;
				}else if(TryToMove(Vector3(0,0,moveVector.z),dt))
				{
					isWalking = true;
				}else if(TryToMove(Vector3(moveVector.x,0,0),dt))
				{
					isWalking = true;
				}
			}


			//update camera
			fppCam->m_vPosition = playerPosition;
			fppCam->m_vView = fppCam->m_vPosition + delta;
			fppCam->m_vVelocity = Vector3(0,0,0);
			fppCam->needUpdate = true;
		}

		if(fppCam->needUpdate)
		{
			mWorld->UpdatePlayerZoneBB(fppCam->m_vPosition);
		}

		//walking sound
		if(isWalking && walkingOnGround && !footInWater)
		{
			if(walkSoundAccu > 0.5f || walkSoundAccu == 0.0f)
			{
				mSoundMgr->PlayWalkSound(soundBlockType);
				walkSoundAccu = 0.0f;
			}
			walkSoundAccu += dt;
		}else
		{
			walkSoundAccu = 0.0f;
		}
	}

	mWorld->UpdateWorldTime(dt);

	if(!mWorld->freezeDayTime)
	{
		//update skydome - every hour
		skyDome->timeOfDay = mWorld->worldDayTime * 0.041666f;
	
		//update sky and sun light time
		//23 000 morning
		//62 500 evening
		//39500 whole day
		//16 normal hours of day
			//2468,75 - hour / 50 seconds(hour in game)
			//49,375
		//8 hours of night
			//4937,5 - hour / 50 seconds
			//98,75 in the night

		if(mWorld->worldDayTime >= 5.0f && mWorld->worldDayTime < 21.0f)
		{
			sunTime += 49.375f * dt;//72
			if(!sunMoonSwitch)//switch to sun texture and reset position
			{
				skyLight->SetTexture(TextureHelper::Instance()->GetTexture(TextureHelper::Sun));
				sunTime = 21600.0f;//6 am
				sunMoonSwitch = true;
			}
		}else
		{
			sunTime += 98.75 * dt;//72
			if(sunMoonSwitch)//switch to sun texture and reset position
			{
				skyLight->SetTexture(TextureHelper::Instance()->GetTexture(TextureHelper::Moon));
				sunTime = 21600.0f;//6 am
				sunMoonSwitch = false;
			}
		}
	}
}

void StatePlay::Draw(StateManager* sManager)
{
	//start rendering
	bool needUpdate = fppCam->needUpdate;
	mRender->StartFrame();

	//draw sky and sun/moon
	{
		sceGumPushMatrix();
		ScePspFVector3 move = {-64,-64,-64};
		sceGumTranslate(&move);
		
		//draw skydome
		skyDome->Render();

		//draw sun/moon
		skyLight->UpdateLightSource(skyLight->TimeToAngle(sunTime));
		skyLight->Render();
		
		sceGumPopMatrix();
	}

	TextureManager::Instance()->SetTextureModulate(texture);

	//draw level

	sceGuFog( 0.0f, 100.75f,GU_COLOR(mWorld->fogColor.x,mWorld->fogColor.y,mWorld->fogColor.z,1.0f));	// Fog parameters
	//sceGuAmbientColor(0xff282828);
	//sceGuFog( 0.0f, 128.75f, 0x00CCCCff );	// Fog parameters
	sceGuEnable(GU_FOG );	// Enable fog

	sceGumPushMatrix();
	mWorld->drawWorld(fppCam->mFrustum,needUpdate);
	sceGumPopMatrix();

	sceGuDisable(GU_FOG );	// disable fog

	if(makeScreen)
	{
		//end frame now to update frame buffer
		mRender->EndFrame();
		//make screenshot
		mRender->TakeNextScreenshot();
		makeScreen = false;
	}

	//render pick cube
	if(showCube)
	{
		//cubePos = fppCam->m_vPosition;
		sceGumPushMatrix();

		ScePspFVector3 move = {cubePos.x,cubePos.y,cubePos.z};
		sceGumTranslate(&move);

		sceGuEnable(GU_BLEND);
		sceGuColor(0xFFFFFFFF);

		//GU_COLOR()
		mRender->Draw(cubeModel);
		sceGuDisable(GU_BLEND);

		sceGumPopMatrix();
	}

	//render cube in right hand
	{
		TextureManager::Instance()->SetTextureModulate(texture);
		sceGumPushMatrix();

		//set view matrix to identity
		sceGumMatrixMode(GU_VIEW);
		sceGumLoadIdentity();

		//translate
		ScePspFVector3 move = {0.47f,-0.32f,-0.7f};
		sceGumTranslate(&move);
		//rotate
		sceGumRotateX(0.2792f);
		sceGumRotateY(0.7853f);
		//scale
		ScePspFVector3 scale = {0.25f,0.25f,0.25f};
		sceGumScale(&scale);

		//draw cube
		sceGuDisable(GU_DEPTH_TEST);
		sceGuDepthMask(1);
		mWorld->drawCubes((selectedCubeSet + selectedCube+1));
		sceGuEnable(GU_DEPTH_TEST);
		sceGuDepthMask(0);

		sceGumMatrixMode(GU_MODEL);
		sceGumPopMatrix();
	}



	//gui
	mRender->SetOrtho(0,0,0,0,0,0);

	if(headInWater)
	{
		sceGuDisable(GU_DEPTH_TEST);
		sceGuDepthMask(1);
		sceGuEnable(GU_BLEND);
		sceGuEnable(GU_TEXTURE_2D);
		sceGuColor(GU_COLOR(1,1,1,0.7f));
		TextureManager::Instance()->SetTextureModulate(blue);
		advancedBlit(0,0,SCR_WIDTH,SCR_HEIGHT,0,0,32);
		sceGuDisable(GU_BLEND);
		sceGuDisable(GU_TEXTURE_2D);
		sceGuEnable(GU_DEPTH_TEST);
		sceGuDepthMask(0);
	}

	//gui
	sceGuDisable(GU_DEPTH_TEST);
	sceGuDepthMask(1);
	sceGuEnable(GU_BLEND);
	sceGuColor(GU_COLOR(1,1,1,1.0f));

	barSprite->Draw();
	selectSprite->Draw();
	crossSprite->Draw();

	sceGuDisable(GU_BLEND);


	//draw 3d cubes on 2d panel
	TextureManager::Instance()->SetTextureModulate(texture);
	int bloStartPos = 100;
	int selectionEnd = 10;
	if(selectedCubeSet == cubesSets * 9)
	{
		selectionEnd = allcubes - (cubesSets * 9) - 1;
	}


	for(int blo = 1 + selectedCubeSet;blo < selectionEnd + selectedCubeSet;blo++)
	{
		sceGumPushMatrix();

		ScePspFVector3 loc = {bloStartPos,253,0.0f};
		sceGumTranslate(&loc);

		ScePspFVector3 sca = {17,17,17.0f};
		sceGumScale(&sca);

		//rotate
		sceGumRotateX(2.61f);
		sceGumRotateY(0.78f);

		sceGuFrontFace(GU_CW);
		mWorld->drawCubes(blo);
		sceGuFrontFace(GU_CCW);

		sceGumPopMatrix();

		bloStartPos+=35;
	}

	sceGuEnable(GU_DEPTH_TEST);
	sceGuDepthMask(0);



	//menu buttons
	if(menuState == 1)
	{
		if(menuOptions)
		{
			sceGuDisable(GU_DEPTH_TEST);
			sceGuEnable(GU_BLEND);
			sceGuColor(GU_COLOR(1,1,1,1.0f));

			buttonSprite->SetPosition(240,40);
			buttonSprite->Draw();

			//fly
			buttonSprite->SetPosition(240,70);
			buttonSprite->Draw();

			//dev menu
			buttonSprite->SetPosition(240,100);
			buttonSprite->Draw();

			//screenshot
			buttonSprite->SetPosition(240,130);
			buttonSprite->Draw();

			//rename map
			buttonSprite->SetPosition(240,160);
			buttonSprite->Draw();

			//player sounds
			buttonSprite->SetPosition(240,190);
			buttonSprite->Draw();

			//menu
			buttonSprite->SetPosition(240,220);
			buttonSprite->Draw();

			//selected button
			sbuttonSprite->SetPosition(240,(optionsMenuPos * 30) + 40);
			sbuttonSprite->Draw();

			sceGuDisable(GU_BLEND);
			sceGuEnable(GU_DEPTH_TEST);

			//draw subtitles on buttons
			canFly == true ? mRender->DebugPrint(240,45,"Fly : ON"): mRender->DebugPrint(240,45,"Fly : OFF");
			devMode == true ? mRender->DebugPrint(240,75,"DevMode : ON"): mRender->DebugPrint(240,75,"DevMode : OFF");
			mRender->DebugPrint(240,105,"Take Screenshot");
			mRender->DebugPrint(240,135,"Change map name");
			mSoundMgr->playerSounds == true ? mRender->DebugPrint(240,165,"Player sounds: ON"):mRender->DebugPrint(240,165,"Player sounds: OFF");
			mWorld->freezeDayTime == true ? mRender->DebugPrint(240,195,"Day time freeze: ON"):mRender->DebugPrint(240,195,"Day time freeze: OFF");
			mRender->DebugPrint(240,225,"Back");
		}else
		{
			sceGuDisable(GU_DEPTH_TEST);
			sceGuEnable(GU_BLEND);
			sceGuColor(GU_COLOR(1,1,1,1.0f));

			//resume
			buttonSprite->SetPosition(240,100);
			buttonSprite->Draw();

			//options
			buttonSprite->SetPosition(240,130);
			buttonSprite->Draw();

			//save
			buttonSprite->SetPosition(240,160);
			buttonSprite->Draw();

			//quit
			buttonSprite->SetPosition(240,190);
			buttonSprite->Draw();

			//quit and save
			buttonSprite->SetPosition(240,220);
			buttonSprite->Draw();

			//selected button
			sbuttonSprite->SetPosition(240,(selectPos * 30) + 100);
			sbuttonSprite->Draw();

			sceGuDisable(GU_BLEND);
			sceGuEnable(GU_DEPTH_TEST);

			//draw subtitles on buttons
			mRender->DebugPrint(240,105,"Resume");
			mRender->DebugPrint(240,135,"Options");
			mRender->DebugPrint(240,165,"Save");
			mRender->DebugPrint(240,195,"Save and Exit");
			mRender->DebugPrint(240,225,"Exit");
		}
	}

	//debug info
	if(devMode)
	{
		mRender->SetFontStyle(0.5f,0xFFFFFFFF,0xFF000000,0x00000000);

		if(dt > 0.0f)
			mRender->DebugPrint(20,20,"fps: %4.2f",(1.0f/dt));
		mRender->DebugPrint(20,30,"cpu: %d%%",mRender->GetCpuUsage());
		mRender->DebugPrint(20,40,"gpu: %d%%",mRender->GetGpuUsage());
		mRender->DebugPrint(20,50,"memory: %d",freeMemory);
		mRender->DebugPrint(20,60,"poly: %d",(mWorld->GetDrawntTrianglesCount() / 3));
		mRender->DebugPrint(20,70,"verts: %d",mWorld->GetDrawntTrianglesCount());
		mRender->DebugPrint(20,80,"day time: %f",mWorld->worldDayTime);
		mRender->DebugPrint(20,90,"sky time: %f",skyDome->timeOfDay);
		mRender->DebugPrint(20,100,"sun time: %f",sunTime);

		mRender->SetFontStyle(0.5f,0xFFFFFFFF,0xFF000000,0x00000200);
	}

	//end frame
	mRender->EndFrame();
}

//additional functions
void StatePlay::advancedBlit(int sx, int sy, int sw, int sh, int dx, int dy, int slice)
{
	int start, end;

	// blit maximizing the use of the texture-cache

	for (start = sx, end = sx+sw; start < end; start += slice, dx += slice)
	{
		TexturesPSPVertex16* vertices = (TexturesPSPVertex16*)sceGuGetMemory(2 * sizeof(TexturesPSPVertex16));
		int width = (start + slice) < end ? slice : end-start;

		vertices[0].u = start; vertices[0].v = sy;
		vertices[0].x = dx; vertices[0].y = dy; vertices[0].z = 0;

		vertices[1].u = start + width; vertices[1].v = sy + sh;
		vertices[1].x = dx + width; vertices[1].y = dy + sh; vertices[1].z = 0;

		sceGuDrawArray(GU_SPRITES,GU_TEXTURE_16BIT|GU_VERTEX_16BIT|GU_TRANSFORM_2D,2,0,vertices);
	}
}

bool StatePlay::TryToMove(Vector3 moveVector,float dt)
{
	// Build a "test vector" that is a little longer than the move vector.
	float moveLength = moveVector.magnitude();
	Vector3 testVector = moveVector;
	testVector.normalize();
	testVector = testVector * (moveLength + 0.2f);

	// Apply this test vector.
	Vector3 movePosition = playerPosition + testVector;
	Vector3 midBodyPoint = movePosition + Vector3(0, -0.7f, 0);
	Vector3 lowerBodyPoint = movePosition + Vector3(0, -1.4f, 0);

	if (!mWorld->SolidAtPointForPlayer(movePosition) && !mWorld->SolidAtPointForPlayer(lowerBodyPoint) && !mWorld->SolidAtPointForPlayer(midBodyPoint))
	{
		playerPosition = playerPosition + moveVector;
		return true;
	}

	//teraz mona robi ciekawe rzeczy

	testVector = moveVector;
	testVector.normalize();
	testVector = testVector * (moveLength + 0.35f);
	movePosition = playerPosition + testVector;
	midBodyPoint = movePosition + Vector3(0, -0.7f, 0);
	lowerBodyPoint = movePosition + Vector3(0, -1.4f, 0);

	unsigned char lowerBlock = mWorld->BlockAtPoint(lowerBodyPoint);
	unsigned char midBlock = mWorld->BlockAtPoint(midBodyPoint);
	unsigned char upperBlock = mWorld->BlockAtPoint(movePosition);

	//kolizja z drabin :>
	if (upperBlock == 6 || lowerBlock == 6 || midBlock == 6)
	{
		playerVelocity.y = CLIMBVELOCITY;
		Vector3 footPosition = playerPosition + Vector3(0.0f, -1.5f, 0.0f);
		if (mWorld->SolidAtPointForPlayer(footPosition))
			playerPosition.y += 0.1f;
		return true;
	}



	return false;
}

bool StatePlay::keyPressed(int currentKey)
{
	//analog reset
	if(analogUp)
	{
		if(mSystemMgr->GetAnalogY() < InputHelper::Instance()->analogYup)
			analogUp = false;
	}
	if(analogDown)
	{
		if(mSystemMgr->GetAnalogY() > InputHelper::Instance()->analogYdown)
			analogDown = false;
	}
	if(analogLeft)
	{
		if(mSystemMgr->GetAnalogX() > InputHelper::Instance()->analogXleft)
			analogLeft = false;
	}
	if(analogRight)
	{
		if(mSystemMgr->GetAnalogX() < InputHelper::Instance()->analogXright)
			analogRight = false;
	}

	//keys
	if(currentKey == 0)//cross
		return mSystemMgr->KeyPressed(PSP_CTRL_UP);
	if(currentKey == 1)//cross
		return mSystemMgr->KeyPressed(PSP_CTRL_DOWN);
	if(currentKey == 2)//cross
		return mSystemMgr->KeyPressed(PSP_CTRL_LEFT);
	if(currentKey == 3)//cross
		return mSystemMgr->KeyPressed(PSP_CTRL_RIGHT);

	if(currentKey == 4)//cross
		return mSystemMgr->KeyPressed(PSP_CTRL_TRIANGLE);
	if(currentKey == 5)//cross
		return mSystemMgr->KeyPressed(PSP_CTRL_CROSS);
	if(currentKey == 6)//cross
		return mSystemMgr->KeyPressed(PSP_CTRL_SQUARE);
	if(currentKey == 7)//cross
		return mSystemMgr->KeyPressed(PSP_CTRL_CIRCLE);

	if(currentKey == 8)//cross
		return mSystemMgr->KeyPressed(PSP_CTRL_LTRIGGER);
	if(currentKey == 9)//cross
		return mSystemMgr->KeyPressed(PSP_CTRL_RTRIGGER);
	if(currentKey == 10)//cross
		return mSystemMgr->KeyPressed(PSP_CTRL_SELECT);
	if(currentKey == 15)//cross
		return mSystemMgr->KeyPressed(PSP_CTRL_START);

	//analog stick....
	if(currentKey == 12)//cross
	{	if(!analogUp)
		{
			if(mSystemMgr->GetAnalogY() > InputHelper::Instance()->analogYup)
			{
				analogUp = true;
				return true;//analog up
			}
		}
	}
	if(currentKey == 11)//cross
	{
		if(!analogDown)
		{
			if(mSystemMgr->GetAnalogY() < InputHelper::Instance()->analogYdown)
			{
				analogDown = true;
				return true;//analog down
			}
		}
	}
	if(currentKey == 13)//cross
	{
		if(!analogLeft)
		{
			if(mSystemMgr->GetAnalogX() < InputHelper::Instance()->analogXleft)
			{
				analogLeft = true;
				return true;//analog left
			}
		}
	}
	if(currentKey == 14)//cross
	{
		if(!analogRight)
		{
			if(mSystemMgr->GetAnalogX() > InputHelper::Instance()->analogXright)
			{
				analogRight = true;
				return true;//analog right
			}
		}
	}

	return false;
}

bool StatePlay::keyHold(int currentKey)
{
	if(currentKey == 0)//cross
		return mSystemMgr->KeyHold(PSP_CTRL_UP);
	if(currentKey == 1)//cross
		return mSystemMgr->KeyHold(PSP_CTRL_DOWN);
	if(currentKey == 2)//cross
		return mSystemMgr->KeyHold(PSP_CTRL_LEFT);
	if(currentKey == 3)//cross
		return mSystemMgr->KeyHold(PSP_CTRL_RIGHT);

	if(currentKey == 4)//cross
		return mSystemMgr->KeyHold(PSP_CTRL_TRIANGLE);
	if(currentKey == 5)//cross
		return mSystemMgr->KeyHold(PSP_CTRL_CROSS);
	if(currentKey == 6)//cross
		return mSystemMgr->KeyHold(PSP_CTRL_SQUARE);
	if(currentKey == 7)//cross
		return mSystemMgr->KeyHold(PSP_CTRL_CIRCLE);

	if(currentKey == 8)//cross
		return mSystemMgr->KeyHold(PSP_CTRL_LTRIGGER);
	if(currentKey == 9)//cross
		return mSystemMgr->KeyHold(PSP_CTRL_RTRIGGER);
	if(currentKey == 10)//cross
		return mSystemMgr->KeyHold(PSP_CTRL_SELECT);
	if(currentKey == 15)//cross
		return mSystemMgr->KeyHold(PSP_CTRL_START);

	//analog stick....
	if(currentKey == 12)//cross
		return (mSystemMgr->GetAnalogY() > InputHelper::Instance()->analogYup);//analog up
	if(currentKey == 11)//cross
		return (mSystemMgr->GetAnalogY() < InputHelper::Instance()->analogYdown);//analog down
	if(currentKey == 13)//cross
		return (mSystemMgr->GetAnalogX() < InputHelper::Instance()->analogXleft);//analog left
	if(currentKey == 14)//cross
		return (mSystemMgr->GetAnalogX() > InputHelper::Instance()->analogXright);//analog right

	return false;
}
